import difflib
import pandas as pd
import xml.etree.ElementTree as ET
import pybliometrics
from pybliometrics.scopus import AbstractRetrieval
from pybliometrics.scopus import SubjectClassifications
from pybliometrics.scopus import SerialTitle
from fuzzywuzzy import fuzz
import glob
import urllib.request, json

pybliometrics.scopus.init()

def is_the_same(name1, name2, thr=80):
    name1 = " ".join(name1.split(",")[::-1]).strip().lower()
    name2 = " ".join(name2.split(",")[::-1]).strip().lower()
    return fuzz.ratio(name1, name2) > thr

def get_students(authors, students):
    return [student for author in authors for student in students if is_the_same(author, student, 78)]

def get_journals(prod):
    pd = prod.find("ARTIGOS-PUBLICADOS")
    if pd is not None:
        return [paper.find("DADOS-BASICOS-DO-ARTIGO").get("DOI")
                for paper in pd.findall("ARTIGO-PUBLICADO")
                if int(paper.find("DADOS-BASICOS-DO-ARTIGO").get("ANO-DO-ARTIGO")) >= MIN_ANO
                ]
    return []

def get_conferences(prod):
    pd = prod.find("TRABALHOS-EM-EVENTOS")
    if pd is not None:
        return [paper.find("DADOS-BASICOS-DO-TRABALHO").get("DOI")
                for paper in pd.findall("TRABALHO-EM-EVENTOS")
                if int(paper.find("DADOS-BASICOS-DO-TRABALHO").get("ANO-DO-TRABALHO")) >= MIN_ANO
                ]
    return []

def get_books(prod, professor, students):
    years, titles, professors, students, chps, scores = [], [], [], [], [], []

    books = prod.find("LIVROS-E-CAPITULOS")

    if books is not None:
        books_p = books.find("LIVROS-PUBLICADOS-OU-ORGANIZADOS")
        if books_p is not None:
            for book in books_p.findall("LIVRO-PUBLICADO-OU-ORGANIZADO"):
                dados = book.find("DADOS-BASICOS-DO-LIVRO")
                year = int(dados.get("ANO"))
                title = dados.get("TITULO-DO-LIVRO")
                authors = [a.get("NOME-COMPLETO-DO-AUTOR") for a in book.findall("AUTORES")]
                student_list = ";".join(get_students(authors, students))

                if year >= MIN_ANO:
                    years.append(year)
                    titles.append(title)
                    professors.append(professor)
                    students.append(student_list)
                    chps.append(False)
                    scores.append(100)

        chapters = books.find("CAPITULOS-DE-LIVROS-PUBLICADOS")
        if chapters is not None:
            for chapter in chapters.findall("CAPITULO-DE-LIVRO-PUBLICADO"):
                dados = chapter.find("DADOS-BASICOS-DO-CAPITULO")
                year = int(dados.get("ANO"))
                title = dados.get("TITULO-DO-CAPITULO-DO-LIVRO")
                authors = [a.get("NOME-COMPLETO-DO-AUTOR") for a in chapter.findall("AUTORES")]
                student_list = ";".join(get_students(authors, students))

                if year >= MIN_ANO:
                    years.append(year)
                    titles.append(title)
                    professors.append(professor)
                    students.append(student_list)
                    chps.append(True)
                    scores.append(25)

    return pd.DataFrame({'Docente' : professors, 'titulo' : titles, 'ano' : years, 'alunos' : students, 'is_chapter' : chps, 'score' : scores })

def get_info_from_paper(doi, professor, students, qualis_conf, is_conf = False):
    if doi == "":
        return None, None, None, None
    try:
        ab = AbstractRetrieval(doi, view="FULL")
        #print("right on: ", doi)
    except:
        #print("error in:", doi)
        return get_info_from_paper_xref(doi, professor, students, qualis_conf, is_conf)
        #print("error in: ", doi)
        #return None, None, None, None
    #try:
    #    ab = AbstractRetrieval(doi, view="FULL")
    #except:
    #    return None, None, None, None

    affiliations = pd.DataFrame(ab.affiliation)
    coauthors = pd.DataFrame(ab.authors)
    fundings = pd.DataFrame(ab.funding)

    authors = [a.given_name + " " + a.surname for a in ab.authors]
    student_list = ";".join(get_students(authors, students))
    year = ab.date_created[0]
    title = ab.title
    issn = ab.issn.print if ab.issn is not None else ab.isbn[0]
    is_conf = True if ab.confname is not None else False
    pubname = ab.publicationName
    try:
        df = pd.DataFrame(SerialTitle(issn, view="CITESCORE").citescoreyearinfolist) if ab.issn is not None else None
        percentiles = [p.percentile for p in df[df.year == SCOPUS_YEAR]['rank'].iloc[0]]
        #print(max(percentiles))
        percentile = max(percentiles) if ab.issn is not None else None #df[df.year == SCOPUS_YEAR].percentcited.values[0] if ab.issn is not None else None
    except:
        percentile = None
    try:
        name = difflib.get_close_matches(ab.confname, qualis_conf['evento'].to_list())[0] if ab.confname is not None else ""
        qualis = qualis_conf[qualis_conf.evento == name].Qualis_Final.values[0] if ab.confname is not None else None
    except:
        print(f"WARNING: {ab.confname} - {difflib.get_close_matches(ab.confname, qualis_conf['evento'].to_list())}")
        qualis = None
    fundings_list = ";".join([f.acronym + "_" + f.funding_id[0] for f in ab.funding if f.acronym is not None and f.funding_id is not None]) if ab.funding is not None else ""
    is_cmp = any(x.abbreviation in ["COMP", "MATH"] for x in ab.subject_areas)

    paper_info = pd.DataFrame({'ano':[year], 'titulo' : [title], 'doi' : [doi], 'fundings' : [fundings_list], 'pubname' : [pubname], 'percentile' : [percentile], 'qualis' : [qualis], 'alunos' : [student_list], 'is_comp' : [is_cmp], 'is_conf' : [is_conf]})

    affiliations['Docente'] = professor
    coauthors['Docente'] = professor
    fundings['Docente'] = professor
    paper_info['Docente'] = professor

    return affiliations, coauthors, fundings, paper_info

def get_info_from_paper_xref(doi, professor, students, qualis_conf, is_conf = False):
    try:
        with urllib.request.urlopen(f"https://api.crossref.org/works/{doi}") as url:
            data = json.load(url)['message']
            if data['type'] == 'journal-article':
                return None, None, None, None

            authors = [a['given'] + " " + a['family'] for a in data['author']]
            student_list = ";".join(get_students(authors, students))
            year = data['indexed']['date-parts'][0][0]
            title = data['title'][0]
            issn = data['ISSN'][0] if 'ISSN' in data else ''
            is_conf = True
            pubname = data['event']['name']
            acname = data['event']['acronym'].split()[0]
            try:
                name = difflib.get_close_matches(acname, qualis_conf['sigla'].to_list())[0] if acname is not None else ""
                qualis = qualis_conf[qualis_conf.sigla == name].Qualis_Final.values[0] if acname is not None else None
            except:
                print(f"WARNING: {acname} - {difflib.get_close_matches(acname, qualis_conf['sigla'].to_list())}")
                qualis = None
            fundings_list = ""
            is_cmp = "COMP"
            paper_info = pd.DataFrame({'ano':[year], 'titulo' : [title], 'doi' : [doi], 'fundings' : [fundings_list], 'pubname' : [pubname], 'percentile' : [0], 'qualis' : [qualis], 'alunos' : [student_list], 'is_comp' : [is_cmp], 'is_conf' : [is_conf]})

            paper_info['Docente'] = professor

        #print(authors, student_list, year, title, issn)
    except:
        #print("unreachable doi:", doi)
        return None, None, None, None

    return None, None, None, paper_info

def concat_none(df1, df2):
    if df1 is None or len(df1) == 0:
        return df2
    if df2 is None or len(df2) == 0:
        return df1
    return pd.concat([df1, df2], ignore_index=True)

# parametros
DTA_CONCLUSAO = '2020-01-01' # data mínima para considerar o egresso
MIN_ANO = 2020 # ano mínimo para considerar publicações
SCOPUS_YEAR = 2023

# Carrega dados de alunos e egressos, convertendo a data de conclusão em datetime
df_alunos = pd.read_csv("alunos.csv")
df_egressos = pd.read_csv("egressos.csv")
df_egressos.CONCLUSAO = pd.to_datetime(df_egressos.CONCLUSAO, dayfirst=True)

alunos = list(df_alunos.NOME.values) + list(df_egressos[df_egressos.CONCLUSAO >= DTA_CONCLUSAO].Nome.values)
qualis_conf = pd.read_csv("qualis-conf - Sheet1.csv")

#print("docente,ano,titulo,issn,doi,fundings,pubname,percentile,qualis,alunos,is_comp,is_conf,is_book")

df_aff = None
df_coauthors = None
df_funds = None
df_papers = None
df_books = None

for fname in glob.glob("cvs/*.xml"):
    tree = ET.parse(fname)
    docente = " ".join(fname.split(".")[0].split("/")[-1].split("_")).upper()

    root = tree.getroot()
    prod = root.find("PRODUCAO-BIBLIOGRAFICA")

    # journals
    journal_papers = get_journals(prod)
    conf_papers = get_conferences(prod)
    df_books = concat_none(df_books, get_books(prod, docente, alunos))

    for paper in journal_papers + conf_papers:
        affiliations, coauthors, fundings, paper_info = get_info_from_paper(paper, docente, alunos, qualis_conf)

        df_aff = concat_none(df_aff, affiliations)
        df_coauthors = concat_none(df_coauthors, coauthors)
        df_funds = concat_none(df_funds, fundings)
        df_papers = concat_none(df_papers, paper_info)


df_aff.to_csv("datasets/affiliations.csv")
df_coauthors.to_csv("datasets/coauthors.csv")
df_funds.to_csv("datasets/fundings.csv")
df_papers.to_csv("datasets/papers.csv")
df_books.to_csv("datasets/books.csv")
